@@ -93,7 +93,8 @@ geary.mc <- function(x, listw, nsim, zero.policy=FALSE,
 	if (is.null(spChk)) spChk <- get.spChkOption()
 	if (spChk && !chkIDs(x, listw))
 		stop("Check of data and weights ID integrity failed")
-	if(nsim > gamma(n+1)) stop("nsim too large for this n")
+        gamres <- suppressWarnings(nsim > gamma(n + 1))
+        if (gamres) stop("nsim too large for this number of observations")
 	if (nsim < 1) stop("non-positive nsim")
 	wc <- spweights.constants(listw, zero.policy)
 	res <- numeric(length=nsim+1)
